@@ -1,27 +1,23 @@
-from decimal import Decimal
-import time
 from datetime import datetime, timedelta
 from dateutil.relativedelta import relativedelta
+import time
 
 from django import forms
-from django.db.models import Q
-from django.conf import settings
 from django.contrib.auth import models as auth_models
 from django.contrib.auth import forms as auth_forms
 from django.core.urlresolvers import reverse
+from django.db.models import Q
 from django.utils.translation import ugettext_lazy as _
-from django.core.exceptions import ValidationError, NON_FIELD_ERRORS
 
 from selectable import forms as selectable_forms
 
-from timepiece.lookups import ProjectLookup, QuickLookup
-from timepiece.lookups import UserLookup, BusinessLookup
-
-from timepiece.models import Project, Entry, Activity, UserProfile, Attribute
-from timepiece.models import ProjectHours
-from timepiece.fields import UserModelChoiceField
 from timepiece import models as timepiece
 from timepiece import utils
+from timepiece.fields import UserModelChoiceField
+from timepiece.lookups import ProjectLookup, QuickLookup
+from timepiece.lookups import UserLookup, BusinessLookup
+from timepiece.models import Project, Entry, Activity, UserProfile, Attribute
+from timepiece.models import ProjectHours
 
 
 class ProjectFiltersForm(forms.Form):
@@ -67,7 +63,7 @@ class CreatePersonForm(auth_forms.UserCreationForm):
         model = auth_models.User
         fields = (
             "username", "first_name", "last_name",
-            "email", "is_active", "is_staff"
+            "email", "is_active", "is_staff", "groups"
         )
 
 
@@ -156,7 +152,7 @@ class QuickSearchForm(forms.Form):
         raise forms.ValidationError('Must be a user, project, or business')
 
 
-class AddUserToProjectForm(forms.Form):
+class SelectUserForm(forms.Form):
     user = selectable_forms.AutoCompleteSelectField(UserLookup, label="")
     user.widget.attrs['placeholder'] = 'Add User'
 
@@ -164,6 +160,14 @@ class AddUserToProjectForm(forms.Form):
         return self.cleaned_data['user']
 
 
+class SelectProjectForm(forms.Form):
+    project = selectable_forms.AutoCompleteSelectField(ProjectLookup, label="")
+    project.widget.attrs['placeholder'] = 'Add Project'
+
+    def save(self):
+        return self.cleaned_data['project']
+
+
 class ClockInForm(forms.ModelForm):
     active_comment = forms.CharField(label='Notes for the active entry',
                                      widget=forms.Textarea, required=False)
@@ -179,11 +183,7 @@ class ClockInForm(forms.ModelForm):
         self.user = kwargs.pop('user')
         self.active = kwargs.pop('active', None)
         initial = kwargs.get('initial', {})
-        default_loc = getattr(
-            settings,
-            'TIMEPIECE_DEFAULT_LOCATION_SLUG',
-            None,
-        )
+        default_loc = utils.get_setting('TIMEPIECE_DEFAULT_LOCATION_SLUG')
         if default_loc:
             try:
                 loc = timepiece.Location.objects.get(slug=default_loc)
@@ -372,16 +372,6 @@ class DateForm(forms.Form):
     to_date = forms.DateField(label="To", required=False,
         input_formats=(DATE_FORMAT,),
         widget=forms.DateInput(format=DATE_FORMAT))
-    status = forms.ChoiceField(choices=STATUS_CHOICES,
-        widget=forms.HiddenInput(), required=False)
-    activity = forms.ModelChoiceField(
-        queryset=timepiece.Activity.objects.all(),
-        widget=forms.HiddenInput(), required=False,
-    )
-    project = forms.ModelChoiceField(
-        queryset=timepiece.Project.objects.all(),
-        widget=forms.HiddenInput(), required=False,
-    )
 
     def clean(self):
         data = self.cleaned_data
@@ -389,7 +379,7 @@ class DateForm(forms.Form):
         to_date = data.get('to_date', None)
         if from_date and to_date and from_date > to_date:
             err_msg = 'The ending date must exceed the beginning date'
-            raise ValidationError(err_msg)
+            raise forms.ValidationError(err_msg)
         return data
 
     def save(self):
@@ -451,15 +441,6 @@ class UserYearMonthForm(YearMonthForm):
         return  (from_date, to_date, self.cleaned_data.get('user', None))
 
 
-class ProjectionForm(DateForm):
-    user = forms.ModelChoiceField(queryset=None)
-
-    def __init__(self, *args, **kwargs):
-        users = kwargs.pop('users')
-        super(ProjectionForm, self).__init__(*args, **kwargs)
-        self.fields['user'].queryset = users
-
-
 class BusinessForm(forms.ModelForm):
     class Meta:
         model = timepiece.Business
@@ -530,6 +511,10 @@ class SearchForm(forms.Form):
     search = forms.CharField(required=False, label='')
     search.widget.attrs['placeholder'] = 'Search'
 
+    def save(self):
+        search = self.cleaned_data.get('search', '')
+        return search
+
 
 class UserForm(forms.ModelForm):
 
@@ -547,12 +532,10 @@ class UserProfileForm(forms.ModelForm):
 
     class Meta:
         model = timepiece.UserProfile
-        exclude = ('user',)
+        exclude = ('user', 'hours_per_week')
 
 
-class ProjectSearchForm(forms.Form):
-    search = forms.CharField(required=False, label='')
-    search.widget.attrs['placeholder'] = 'Search'
+class ProjectSearchForm(SearchForm):
     status = forms.ChoiceField(required=False, choices=[], label='')
 
     def __init__(self, *args, **kwargs):
@@ -655,3 +638,14 @@ class ProjectHoursForm(forms.ModelForm):
 
     class Meta:
         model = ProjectHours
+
+
+class ProductivityReportForm(forms.Form):
+    DATE_FORMAT = '%Y-%m-%d'
+    ORGANIZE_BY_CHOICES = (
+        ('week', 'Week'),
+        ('person', 'People'),
+    )
+    project = selectable_forms.AutoCompleteSelectField(ProjectLookup)
+    organize_by = forms.ChoiceField(choices=ORGANIZE_BY_CHOICES,
+            widget=forms.RadioSelect(), initial=ORGANIZE_BY_CHOICES[0][0])
