@@ -1,12 +1,12 @@
 import datetime
-import logging
+from dateutil.relativedelta import relativedelta
 from decimal import Decimal
+import logging
 
-from django.conf import settings
 from django.contrib.auth.models import User
-from django.core.exceptions import ValidationError, NON_FIELD_ERRORS
+from django.core.exceptions import ValidationError
 from django.db import models
-from django.db.models import Q, Avg, Sum, Max, Min
+from django.db.models import Q, Sum, Max, Min
 
 try:
     from django.utils import timezone
@@ -15,11 +15,6 @@ except ImportError:
 
 from timepiece import utils
 
-from dateutil.relativedelta import relativedelta
-from dateutil import rrule
-
-from datetime import timedelta
-
 
 class Attribute(models.Model):
     ATTRIBUTE_TYPES = (
@@ -117,9 +112,6 @@ class Project(models.Model):
     def __unicode__(self):
         return self.name
 
-    def trac_url(self):
-        return settings.TRAC_URL % self.tracker_url
-
 
 class RelationshipType(models.Model):
     name = models.CharField(max_length=255, unique=True)
@@ -293,7 +285,7 @@ class EntryQuerySet(models.query.QuerySet):
                                                       'date')
         return qs
 
-    def timespan(self, from_date, to_date=None, span=None):
+    def timespan(self, from_date, to_date=None, span=None, current=False):
         """
         Takes a beginning date a filters entries. An optional to_date can be
         specified, or a span, which is one of ('month', 'week', 'day').
@@ -309,10 +301,9 @@ class EntryQuerySet(models.query.QuerySet):
                 diff = relativedelta(days=1)
             if diff is not None:
                 to_date = from_date + diff
-
-        datesQ = Q()
-        datesQ &= Q(end_time__gte=from_date)
+        datesQ = Q(end_time__gte=from_date)
         datesQ &= Q(end_time__lt=to_date) if to_date else Q()
+        datesQ |= Q(end_time__isnull=True) if current else Q()
         return self.filter(datesQ)
 
 
@@ -343,7 +334,7 @@ class EntryWorkedManager(models.Manager):
 
     def get_query_set(self):
         qs = EntryQuerySet(self.model)
-        projects = getattr(settings, 'TIMEPIECE_PROJECTS', {})
+        projects = utils.get_setting('TIMEPIECE_PAID_LEAVE_PROJECTS')
         return qs.exclude(project__in=projects.values())
 
 
@@ -563,7 +554,17 @@ class Entry(models.Model):
 
         return seconds + (delta.days * 86400)
 
-    def __total_hours(self):
+    def get_active_seconds(self):
+        """Use with active entries to obtain the seconds worked so far."""
+        if not self.end_time:
+            if self.is_paused:
+                self.end_time = self.pause_time
+            else:
+                self.end_time = timezone.now()
+        return self.get_seconds()
+
+    @property
+    def total_hours(self):
         """
         Determined the total number of hours worked in this entry
         """
@@ -572,14 +573,13 @@ class Entry(models.Model):
         if total < 0:
             total = 0
         return total
-    total_hours = property(__total_hours)
 
-    def __is_paused(self):
+    @property
+    def is_paused(self):
         """
         Determine whether or not this entry is paused
         """
         return bool(self.pause_time)
-    is_paused = property(__is_paused)
 
     def pause(self):
         """
@@ -616,12 +616,12 @@ class Entry(models.Model):
         else:
             self.pause()
 
-    def __is_closed(self):
+    @property
+    def is_closed(self):
         """
         Determine whether this entry has been closed or not
         """
         return bool(self.end_time)
-    is_closed = property(__is_closed)
 
     def clock_in(self, user, project):
         """
@@ -633,11 +633,12 @@ class Entry(models.Model):
             if not self.start_time:
                 self.start_time = timezone.now()
 
-    def __is_editable(self):
+    @property
+    def is_editable(self):
         return self.status == 'unverified'
-    is_editable = property(__is_editable)
 
-    def __delete_key(self):
+    @property
+    def delete_key(self):
         """
         Make it a little more interesting for deleting logs
         """
@@ -651,18 +652,18 @@ class Entry(models.Model):
         else:
             key = hashlib.sha1(salt).hexdigest()
         return key
-    delete_key = property(__delete_key)
 
     @staticmethod
     def summary(user, date, end_date):
         """
         Returns a summary of hours worked in the given time frame, for this
-        user.  The setting TIMEPIECE_PROJECTS can be used to separate out hours
-        for paid leave that should not be included in the total worked (e.g.,
-        sick time, vacation time, etc.).  Those hours will be added to the
-        summary separately using the dictionary key set in TIMEPIECE_PROJECTS.
+        user.  The setting TIMEPIECE_PAID_LEAVE_PROJECTS can be used to
+        separate out hours for paid leave that should not be included in the
+        total worked (e.g., sick time, vacation time, etc.).  Those hours will
+        be added to the summary separately using the dictionary key set in
+        TIMEPIECE_PAID_LEAVE_PROJECTS.
         """
-        projects = getattr(settings, 'TIMEPIECE_PROJECTS', {})
+        projects = utils.get_setting('TIMEPIECE_PAID_LEAVE_PROJECTS')
         entries = user.timepiece_entries.filter(
             end_time__gt=date, end_time__lt=end_date)
         data = {
@@ -704,6 +705,7 @@ class Entry(models.Model):
         return '%s on %s' % (self.user, self.project)
 
     class Meta:
+        ordering = ('-start_time',)
         verbose_name_plural = 'entries'
         permissions = (
             ('can_clock_in', 'Can use Pendulum to clock in'),
@@ -785,12 +787,6 @@ class ProjectContract(models.Model):
               self.assignments.aggregate(sum=Sum('num_hours'))['sum']
         return self._hours_assigned or 0
 
-    @property
-    def hours_allocated(self):
-        allocations = AssignmentAllocation.objects.filter(
-            assignment__contract=self)
-        return allocations.aggregate(sum=Sum('hours'))['sum']
-
     @property
     def hours_remaining(self):
         return self.num_hours - self.hours_worked()
@@ -948,68 +944,6 @@ class ContractAssignment(models.Model):
         type_list = ['ending', 'starting', 'ongoing', ]
         return type_list[self.this_weeks_priority_number]
 
-    def get_average_weekly_committment(self):
-        week_start = utils.get_week_start()
-        # calculate hours left on contract (subtract worked hours this week)
-        remaining = self.num_hours - self._filtered_hours_worked(week_start)
-        commitment = remaining / self.contract.weeks_remaining.count()
-        return commitment
-
-    def weekly_commitment(self, day=None):
-        self._log("Commitment for {0}".format(day))
-        # earlier assignments may have already allocated time for this week
-        unallocated = self.unallocated_hours_for_week(day)
-        self._log('Unallocated hours {0}'.format(unallocated))
-        reserved = self.remaining_min_hours()
-        self._log('Reserved hours {0}'.format(reserved))
-        # start with unallocated hours
-        commitment = unallocated
-        # reserve required hours on later assignments (min_hours_per_week)
-        commitment -= self.remaining_min_hours()
-        self._log('Commitment after reservation {0}'.format(commitment))
-        # if we're under the needed minimum hours and we have available
-        # time, then raise our commitment to the desired level
-        if commitment < self.min_hours_per_week \
-        and unallocated >= self.min_hours_per_week:
-            commitment = self.min_hours_per_week
-        self._log('Commitment after minimum weekly hours {0}'\
-            .format(commitment))
-        # calculate hours left on contract (subtract worked hours this week)
-        week_start = utils.get_week_start(day)
-        remaining = self.num_hours - self._filtered_hours_worked(week_start)
-        total_allocated = self.blocks.aggregate(s=Sum('hours'))['s'] or 0
-        remaining -= total_allocated
-        if remaining < 0:
-            remaining = 0
-        self._log('Remaining {0}'.format(remaining))
-        # reduce commitment to remaining hours
-        if commitment > remaining:
-            commitment = remaining
-        self._log('Final commitment {0}'.format(commitment))
-        return commitment
-
-    def allocated_hours_for_week(self, day):
-        week, next_week = utils.get_week_window(day)
-        allocs = AssignmentAllocation.objects
-        allocs = allocs.filter(assignment__user=self.user)
-        allocs = allocs.filter(date__gte=week, date__lt=next_week)
-        hours = allocs.aggregate(s=Sum('hours'))['s']
-        return hours or 0
-
-    def unallocated_hours_for_week(self, day):
-        """ Calculate number of hours left to work for a week """
-        allocated = self.allocated_hours_for_week(day)
-        self._log('Allocated hours {0}'.format(allocated))
-        try:
-            schedule = PersonSchedule.objects.filter(user=self.user)[0]
-        except IndexError:
-            schedule = None
-        if schedule:
-            unallocated = schedule.hours_per_week - allocated
-        else:
-            unallocated = 40 - allocated
-        return unallocated
-
     def remaining_contracts(self):
         assignments = ContractAssignment.objects.exclude(pk=self.pk)
         assignments = assignments.filter(end_date__gte=self.end_date,
@@ -1027,79 +961,10 @@ class ContractAssignment(models.Model):
         return u'%s / %s' % (self.user, self.contract.project)
 
 
-class AllocationManager(models.Manager):
-
-    def during_this_week(self, user, day=None):
-        week = utils.get_week_start(day=day)
-        return self.get_query_set().filter(
-            date=week, assignment__user=user,
-            assignment__contract__status='current'
-            ).exclude(hours=0)
-
-
-class AssignmentAllocation(models.Model):
-    assignment = models.ForeignKey(ContractAssignment, related_name='blocks')
-    date = models.DateField()
-    hours = models.DecimalField(max_digits=8, decimal_places=2, default=0)
-
-    @property
-    def hours_worked(self):
-        if not hasattr(self, '_hours_worked'):
-            end_date = self.date + datetime.timedelta(weeks=1)
-            self._hours_worked = self.assignment.\
-                    filtered_hours_worked_with_in_window(self.date, end_date)
-        return self._hours_worked or 0
-
-    @property
-    def hours_left(self):
-        if not hasattr(self, '_hours_left'):
-            self._hours_left = self.hours - self.hours_worked
-        return self._hours_left or 0
-
-    objects = AllocationManager()
-
-
-class PersonSchedule(models.Model):
-    user = models.ForeignKey(
-        User,
-        unique=True,
-        null=True,
-    )
-    hours_per_week = models.DecimalField(max_digits=8, decimal_places=2,
-                                         default=0)
-    end_date = models.DateField()
-
-    @property
-    def furthest_end_date(self):
-        assignments = self.user.assignments.order_by('-end_date')
-        assignments = assignments.exclude(contract__status='complete')
-        try:
-            end_date = assignments.values('end_date')[0]['end_date']
-        except IndexError:
-            end_date = self.end_date
-        return end_date
-
-    @property
-    def hours_available(self):
-        today = datetime.date.today()
-        weeks_remaining = (self.end_date - today).days / 7.0
-        return float(self.hours_per_week) * weeks_remaining
-
-    @property
-    def hours_scheduled(self):
-        if not hasattr(self, '_hours_scheduled'):
-            self._hours_scheduled = 0
-            now = timezone.now()
-            for assignment in self.user.assignments.filter(end_date__gte=now):
-                self._hours_scheduled += assignment.hours_remaining
-        return self._hours_scheduled
-
-    def __unicode__(self):
-        return unicode(self.user)
-
-
 class UserProfile(models.Model):
     user = models.OneToOneField(User, unique=True, related_name='profile')
+    hours_per_week = models.DecimalField(max_digits=8, decimal_places=2,
+                                         default=40)
 
     def __unicode__(self):
         return unicode(self.user)
