@@ -8,7 +8,7 @@ from decimal import Decimal
 from django.test import TestCase
 from django.core.urlresolvers import reverse
 from django.template.defaultfilters import slugify
-from django.contrib.auth.models import User, Permission
+from django.contrib.auth.models import User, Permission, Group
 from django.contrib.contenttypes.models import ContentType
 from django.db.models import Q
 
@@ -22,7 +22,8 @@ from timepiece import utils
 
 
 class TimepieceDataTestCase(TestCase):
-    def create_business(self, data={}):
+    def create_business(self, data=None):
+        data = data or {}
         name = self.random_string(30, extra_chars=' ')
         defaults = {
             'name': name,
@@ -34,17 +35,8 @@ class TimepieceDataTestCase(TestCase):
         chars = string.letters + extra_chars
         return ''.join([random.choice(chars) for i in range(length)])
 
-    def create_person(self, data={}):
-        first_name = self.random_string(20)
-        last_name = self.random_string(20)
-        defaults = {
-            'first_name': first_name,
-            'last_name': last_name,
-        }
-        defaults.update(data)
-        return User.objects.create(**defaults)
-
-    def create_project_type(self, data={}):
+    def create_project_type(self, data=None):
+        data = data or {}
         defaults = {
             'label': self.random_string(30, extra_chars=' '),
             'type': 'project-type',
@@ -52,7 +44,8 @@ class TimepieceDataTestCase(TestCase):
         defaults.update(data)
         return timepiece.Attribute.objects.create(**defaults)
 
-    def create_project_status(self, data={}):
+    def create_project_status(self, data=None):
+        data = data or {}
         defaults = {
             'label': self.random_string(24, extra_chars=' '),
             'type': 'project-status',
@@ -60,7 +53,8 @@ class TimepieceDataTestCase(TestCase):
         defaults.update(data)
         return timepiece.Attribute.objects.create(**defaults)
 
-    def create_project(self, billable=False, name=None, data={}):
+    def create_project(self, billable=False, name=None, data=None):
+        data = data or {}
         if not name:
             name = self.random_string(30, extra_chars=' ')
         defaults = {
@@ -73,25 +67,24 @@ class TimepieceDataTestCase(TestCase):
         if 'business' not in defaults:
             defaults['business'] = self.create_business()
         if 'point_person' not in defaults:
-            defaults['point_person'] = User.objects.create_user(
-                self.random_string(10),
-                'test@example.com',
-                'test',
-            )
+            defaults['point_person'] = self.create_user()
         return timepiece.Project.objects.create(**defaults)
 
-    def create_project_relationship(self, types=[], data={}):
+    def create_project_relationship(self, types=None, data=None):
+        types = types or []
+        data = data or {}
         defaults = {}
         defaults.update(data)
         if 'user' not in defaults:
-            defaults['user'] = self.create_person()
+            defaults['user'] = self.create_user()
         if 'project' not in defaults:
             defaults['project'] = self.create_project()
         relationship = timepiece.ProjectRelationship.objects.create(**defaults)
         relationship.types.add(*types)
         return relationship
 
-    def create_relationship_type(self, data={}):
+    def create_relationship_type(self, data=None):
+        data = data or {}
         defaults = {
             'name': self.random_string(25),
             'slug': self.random_string(25),
@@ -99,7 +92,9 @@ class TimepieceDataTestCase(TestCase):
         defaults.update(data)
         return timepiece.RelationshipType.objects.create(**defaults)
 
-    def create_activity(self, activity_groups=[], data={}):
+    def create_activity(self, activity_groups=None, data=None):
+        activity_groups = activity_groups or []
+        data = data or {}
         defaults = {
             'code': self.random_string(5, extra_chars=' '),
             'name': self.random_string(50, extra_chars=' '),
@@ -112,7 +107,8 @@ class TimepieceDataTestCase(TestCase):
             activity.save()
         return activity
 
-    def create_location(self, data={}):
+    def create_location(self, data=None):
+        data = data or {}
         defaults = {
             'name': self.random_string(255, extra_chars=' '),
             'slug': self.random_string(255),
@@ -120,7 +116,8 @@ class TimepieceDataTestCase(TestCase):
         defaults.update(data)
         return timepiece.Location.objects.create(**defaults)
 
-    def create_entry(self, data={}):
+    def create_entry(self, data=None):
+        data = data or {}
         defaults = {}
         defaults.update(data)
         if 'user' not in defaults:
@@ -135,7 +132,8 @@ class TimepieceDataTestCase(TestCase):
             defaults['status'] = 'unverified'
         return timepiece.Entry.objects.create(**defaults)
 
-    def create_project_contract(self, data={}):
+    def create_project_contract(self, data=None):
+        data = data or {}
         defaults = {
             'start_date': datetime.date.today(),
             'end_date': datetime.date.today() + datetime.timedelta(weeks=2),
@@ -147,27 +145,18 @@ class TimepieceDataTestCase(TestCase):
             defaults['project'] = self.create_project()
         return timepiece.ProjectContract.objects.create(**defaults)
 
-    def create_contract_assignment(self, data={}):
+    def create_contract_assignment(self, data=None):
+        data = data or {}
         defaults = {}
         defaults.update(data)
         if 'user' not in defaults:
-            user = self.create_person()
+            user = self.create_user()
         if 'contract' not in defaults:
             defaults['contract'] = self.create_project()
         defaults['start_date'] = defaults['contract'].start_date
         defaults['end_date'] = defaults['contract'].end_date
         return timepiece.ContractAssignment.objects.create(**defaults)
 
-    def create_person_schedule(self, data={}):
-        defaults = {
-            'hours_per_week': 40,
-            'end_date': datetime.date.today() + datetime.timedelta(weeks=2),
-        }
-        defaults.update(data)
-        if 'user' not in defaults:
-            defaults['user'] = self.create_person()
-        return timepiece.PersonSchedule.objects.create(**defaults)
-
     def log_time(self, delta=None, billable=True, project=None,
         start=None, end=None, status=None, pause=0, activity=None, user=None):
         if not user:
@@ -205,17 +194,32 @@ class TimepieceDataTestCase(TestCase):
         return self.create_entry(data)
 
     def create_user(self, username=None, email=None, password=None,
-            is_superuser=False):
+            user_permissions=None, groups=None, **kwargs):
         username = self.random_string(25) if not username else username
         email = self.random_string(10) + "@example.com" if not email else email
         password = self.random_string(25) if not password else password
         user = User.objects.create_user(username, email, password)
-        if is_superuser:
-            user.is_superuser = True
-            user.save()
-        return user
+        if user_permissions:
+            user.user_permissions = user_permissions
+        if groups:
+            user.groups = groups
+        if kwargs:
+            User.objects.filter(pk=user.pk).update(**kwargs)
+        return User.objects.get(pk=user.pk)
 
-    def create_activity_group(self, name=None, data={}):
+    def create_auth_group(self, permissions=None, **kwargs):
+        defaults = {
+            'name': self.random_string(25),
+        }
+        defaults.update(kwargs)
+        group = Group.objects.create(**defaults)
+        if permissions:
+            group.permissions = permissions
+            group.save()
+        return group
+
+    def create_activity_group(self, name=None, data=None):
+        data = data or {}
         defaults = {
             'name': name or self.random_string(25),
         }
@@ -223,23 +227,21 @@ class TimepieceDataTestCase(TestCase):
         return timepiece.ActivityGroup.objects.create(**defaults)
 
     def create_project_hours_entry(self, week_start=None, project=None,
-                user=None, hours=None, published=None):
-        week_start = (utils.get_week_start(add_tzinfo=False)
-            if not week_start else week_start)
-        project = self.create_project() if not project else project
-        user = self.create_user() if not user else user
-        hours = Decimal(str(random.random() * 20)) if not hours else hours
+                user=None, hours=None, **kwargs):
+        week_start = week_start or utils.get_week_start(add_tzinfo=False)
+        project = project or self.create_project()
+        user = user or self.create_user()
+        hours = Decimal(str(random.random() * 20)) if hours is None else hours
         return timepiece.ProjectHours.objects.create(week_start=week_start,
-                project=project, user=user, hours=hours,
-                published=published or False)
+                project=project, user=user, hours=hours, **kwargs)
 
     def setUp(self):
-        self.user = User.objects.create_user('user', 'u@abc.com', 'abc')
+        self.user = self.create_user('user', 'u@abc.com', 'abc')
         self.user.last_name = 'Jones'
-        self.user2 = User.objects.create_user('user2', 'u2@abc.com', 'abc')
+        self.user2 = self.create_user('user2', 'u2@abc.com', 'abc')
         self.user2.last_name = 'Smith'
-        self.superuser = User.objects.create_user('superuser',
-                                                  'super@abc.com', 'abc')
+        self.superuser = self.create_user('superuser', 'super@abc.com', 'abc',
+                is_superuser=True)
         permissions = Permission.objects.filter(
             content_type=ContentType.objects.get_for_model(timepiece.Entry),
             codename__in=('can_clock_in', 'can_clock_out',
@@ -247,8 +249,6 @@ class TimepieceDataTestCase(TestCase):
         )
         self.user.user_permissions = permissions
         self.user2.user_permissions = permissions
-        self.superuser.is_superuser = True
-        self.superuser.save()
         self.user.save()
         self.user2.save()
         self.activity = timepiece.Activity.objects.create(
