@@ -45,41 +45,6 @@ class ProjectTestCase(TimepieceDataTestCase):
         self.log_time(project=self.p1, start=days[4], delta=(1, 0))
         self.log_time(project=self.p2, start=days[4], delta=(1, 0))
 
-    def test_remove_user(self):
-        self.user.is_superuser = True
-        self.user.save()
-        self.client.login(username=self.user.username, password='abc')
-        self.assertEquals(self.project.users.all().count(), 1)
-        url = reverse('remove_user_from_project',
-            args=(self.project.pk, self.user.pk,))
-        response = self.client.get(url, {'next': '/'})
-        self.assertEquals(response.status_code, 302)
-        self.assertEquals(self.project.users.all().count(), 0)
-
-    def test_add_user(self):
-        self.user.is_superuser = True
-        self.user.save()
-
-        self.client.login(username=self.user.username, password='abc')
-        timepiece.ProjectRelationship.objects.all().delete()
-        self.assertEquals(self.project.users.all().count(), 0)
-        url = reverse('add_user_to_project', args=(self.project.pk,))
-        response = self.client.post(url, {
-            'user_0': self.user.get_full_name(),  # Can be anything
-            'user_1': self.user.pk  # Must be pk
-        })
-        self.assertEquals(response.status_code, 302)
-        self.assertEquals(self.project.users.all().count(), 1)
-
-        timepiece.ProjectRelationship.objects.all().delete()
-        self.assertEquals(self.project.users.all().count(), 0)
-        response = self.client.post(url, {
-            'user_0': '',  # Can be anything
-            'user_1': self.user.pk  # Must be pk
-        })
-        self.assertEquals(response.status_code, 302)
-        self.assertEquals(self.project.users.all().count(), 1)
-
     def testNoPermission(self):
         self.client.login(username='user', password='abc')
         self.make_entries()
