@@ -3,27 +3,36 @@ from dateutil import rrule
 from dateutil.relativedelta import relativedelta
 from decimal import Decimal
 from itertools import groupby
-import time
-import calendar
 
-from django.http import HttpResponse, HttpResponseRedirect, Http404
-from django.shortcuts import render_to_response
-from django.template import RequestContext
-from django.template.defaultfilters import slugify
-from django.db.models import Sum, get_model
-from django.contrib.sites.models import Site
-from django.utils.functional import lazy
+from django.conf import settings
 from django.core.urlresolvers import reverse
+from django.db.models import Sum, get_model, Q
+from django.template.defaultfilters import slugify
+from django.utils.functional import lazy
 
 try:
     from django.utils import timezone
 except ImportError:
     from timepiece import timezone
 
+from timepiece.defaults import TimepieceDefaults
+
 
 reverse_lazy = lazy(reverse, str)
 
 
+defaults = TimepieceDefaults()
+def get_setting(name, **kwargs):
+    if hasattr(settings, name):
+        return getattr(settings, name)
+    if hasattr(kwargs, 'default'):
+        return kwargs['default']
+    if hasattr(defaults, name):
+        return getattr(defaults, name)
+    msg = '{0} must be specified in your project settings.'.format(name)
+    raise AttributeError(msg)
+
+
 def slugify_uniquely(s, queryset=None, field='slug'):
     """
     Returns a slug based on 's' that is unique for all instances of the given
@@ -44,51 +53,6 @@ def slugify_uniquely(s, queryset=None, field='slug'):
     return new_slug
 
 
-def render_with(template_name):
-    """
-    Renders the view wrapped by this decorator with the given template.  The
-    view should return the context to be used in the template, or an
-    HttpResponse.
-
-    If the view returns an HttpResponseRedirect, the decorator will redirect
-    to the given URL, or to request.REQUEST['next'] (if it exists).
-    """
-    def render_with_decorator(view_func):
-        def wrapper(*args, **kwargs):
-            request = args[0]
-            response = view_func(*args, **kwargs)
-
-            if isinstance(response, HttpResponse):
-                if isinstance(response, HttpResponseRedirect) and \
-                  'next' in request.REQUEST:
-                    return HttpResponseRedirect(request.REQUEST['next'])
-                else:
-                    return response
-            else:
-                # assume response is a context dictionary
-                context = response
-                return render_to_response(
-                    template_name,
-                    context,
-                    context_instance=RequestContext(request),
-                )
-        return wrapper
-    return render_with_decorator
-
-
-DEFAULT_TIME_FORMATS = [
-    '%H:%M',        # 23:15         => 23:15:00
-    '%H:%M:%S',     # 05:50:21      => 05:50:21
-    '%I:%M:%S %p',  # 11:40:53 PM   => 23:40:53
-    '%I:%M %p',     # 6:21 AM       => 06:21:00
-    '%I %p',        # 1 pm          => 13:00:00
-    '%I:%M:%S%p',   # 8:45:52pm     => 23:45:52
-    '%I:%M%p',      # 12:03am       => 00:03:00
-    '%I%p',         # 12pm          => 12:00:00
-    '%H',           # 22            => 22:00:00
-]
-
-
 def add_timezone(value, tz=None):
     """If the value is naive, then the timezone is added to it.
 
@@ -105,29 +69,6 @@ def add_timezone(value, tz=None):
     return value
 
 
-def parse_time(time_str, input_formats=None):
-    """
-    This function will take a string with some sort of representation of time
-    in it.  The string will be parsed using a variety of formats until a match
-    is found for the format given.  The return value is a datetime.time object.
-    """
-    formats = input_formats or DEFAULT_TIME_FORMATS
-
-    # iterate over all formats until we find a match
-    for format in formats:
-        try:
-            # attempt to parse the time with the current format
-            value = time.strptime(time_str, format)
-        except ValueError:
-            continue
-        else:
-            # turn the time_struct into a datetime.time object
-            return add_timezone(datetime.time(*value[3:6]))
-
-    # return None if there's no matching format
-    return None
-
-
 def get_total_time(seconds):
     """
     Returns the specified number of seconds in an easy-to-read HH:MM:SS format
@@ -186,28 +127,6 @@ def get_week_window(day):
     return list(weeks)
 
 
-def date_filter(func):
-    def inner_decorator(request, *args, **kwargs):
-        from timepiece import forms as timepiece_forms
-        if 'to_date' in request.GET:
-            form = timepiece_forms.DateForm(request.GET)
-            if form.is_valid():
-                from_date, to_date = form.save()
-                status = form.cleaned_data.get('status')
-                activity = form.cleaned_data.get('activity')
-            else:
-                raise Http404
-        else:
-            form = timepiece_forms.DateForm()
-            today = datetime.date.today()
-            from_date = today.replace(day=1)
-            to_date = from_date + relativedelta(months=1)
-            status = activity = None
-        return func(request, form, from_date, to_date, status, activity,
-            *args, **kwargs)
-    return inner_decorator
-
-
 def get_hours(entries):
     hours = {
         'total': 0,
@@ -249,7 +168,8 @@ def grouped_totals(entries):
                                                         'project__name')
     weeks = {}
     for week, week_entries in groupby(weekly, lambda x: x['date']):
-        week = add_timezone(week)
+        if week is not None:
+            week = add_timezone(week)
         weeks[week] = get_hours(week_entries)
     days = []
     last_week = None
@@ -500,3 +420,62 @@ def process_dates(dates, start=None, end=None):
         if to_date else end
 
     return from_date, to_date
+
+
+def get_total_seconds(td):
+    """
+    The equivalent for datetime.timedelta.total_seconds() for Python 2.6
+    """
+    if hasattr(td, 'total_seconds'):
+        return td.total_seconds()
+    return (td.microseconds + (td.seconds + td.days * 24 * 3600) * 1e6) / 1e6
+
+
+def get_hours_per_week(user):
+    """Retrieves the number of hours the user should work per week."""
+    UserProfile = get_model('timepiece', 'UserProfile')
+    try:
+        profile = UserProfile.objects.get(user=user)
+    except UserProfile.DoesNotExist:
+        profile = None
+    return profile.hours_per_week if profile else Decimal('40.00')
+
+
+def process_progress(entries, assignments):
+    """
+    Returns a list of progress summary data (pk, name, hours worked, and hours
+    assigned) for each project either worked or assigned.
+    The list is ordered by project name.
+    """
+    Project = get_model('timepiece', 'Project')
+    ProjectHours = get_model('timepiece', 'ProjectHours')
+
+    # Determine all projects either worked or assigned.
+    project_q = Q(id__in=assignments.values_list('project__id', flat=True))
+    project_q |= Q(id__in=entries.values_list('project__id', flat=True))
+    projects = Project.objects.filter(project_q).values('pk', 'name')
+
+    # Hours per project.
+    project_data = {}
+    for project in projects:
+        try:
+            assigned = assignments.get(project__id=project['pk']).hours
+        except ProjectHours.DoesNotExist:
+            assigned = Decimal('0.00')
+        project_data[project['pk']] = {
+            'pk': project['pk'],
+            'name': project['name'],
+            'assigned': assigned,
+            'worked': Decimal('0.00'),
+        }
+
+    for entry in entries:
+        pk = entry.project_id
+        hours = Decimal('%.2f' % round(entry.get_active_seconds() / 3600.0, 2))
+        project_data[pk]['worked'] += hours
+
+    # Sort by maximum of worked or assigned hours (highest first).
+    key = lambda x: x['name'].lower()
+    project_progress = sorted(project_data.values(), key=key)
+
+    return project_progress
